using System;
using System.IO;
using System.Collections;
using System.Configuration;
using System.Data;
using System.Diagnostics;
using System.Text;
using System.Web;
using System.Collections.Generic;

using System.Xml;
using System.Xml.XPath;

namespace MetX.Urn
{

	/// <summary>This class is automatically made available as urn:xlg while rendering xsl pages from any of the MetX.Web xsl rendering classes. Each function provides some string, date, and totaling capability as well as some basic variable storage that can survive template calls.
	/// <para>NOTE: XSL functions must return a variable. Most functions in this library normally wouldn't return anything, but to accomodate XSL, they return a blank string.</para>
	/// </summary>
	public class xlgUrn
	{
		private int NextID;
		private int NextHash = 1;
		private int NextLayer = 10000;
		private string CurrRowClass = "contentDataRow2";
		private string NextRowClass = "contentDataRow1";
		private Dictionary<string, double> RunningTotals;
		private Dictionary<string, string> HT;
		private Dictionary<string, string> Vars;
		private Dictionary<string, StringBuilder> sbVars;
		private string SameAsLast_LastItem = string.Empty;

		private string ThemePath = "~/theme/default/";
		private string DefaultThemePath = "~/theme/default/";
		private string SupportPath = "/xlgSupport/";

		HttpContext c;


		/// <summary>
		/// Returns if one or more bits are set in ToCheck
		/// </summary>
		/// <param name="ToCheck">The Integer to check</param>
		/// <param name="Mask">The bit mask to check</param>
		/// <returns>true if all bits set in Mask are set in ToCheck, else false</returns>
		public bool HasBit(int ToCheck, int Mask)
		{
			return (ToCheck & Mask) != 0;
		}

		/// <summary>Sets the internal variable 'ThemePath' used with FilePath, FileUrl, and FileContents</summary>
		/// <param name="ThemePath">The virtual path to the theme directory (such as '/~/theme/YourClientName/')</param>
		/// <param name="DefaultThemePath">The virtual path to the defalt theme directory (such as '/~/theme/default/')</param>
		/// <returns>A blank string</returns>
		public string SetThemePath(string ThemePath, string DefaultThemePath)
		{
			if (!ThemePath.EndsWith("/"))
				ThemePath += "/";
			if (!DefaultThemePath.EndsWith("/"))
				DefaultThemePath += "/";
			this.ThemePath = ThemePath;
			this.DefaultThemePath = DefaultThemePath;
			return string.Empty;
		}

		/// <summary>Sets the internal variable 'Support' used with FilePath, FileUrl, and FileContents</summary>
		/// <param name="SupportPath">The virtual path to the xlgSupport directory (defalts to '/xlgSupport/')</param>
		/// <returns>A blank string</returns>
		public string UrlEncode(string ToEncode)
		{
			if (ToEncode == null)
				return string.Empty;
			return System.Web.HttpUtility.UrlEncode(ToEncode);
		}
		/// <summary>Sets the internal variable 'Support' used with FilePath, FileUrl, and FileContents</summary>
		/// <param name="SupportPath">The virtual path to the xlgSupport directory (defalts to '/xlgSupport/')</param>
		/// <returns>A blank string</returns>
		public string SetSupportPath(string SupportPath)
		{
			if (!SupportPath.EndsWith("/"))
				SupportPath += "/";
			this.SupportPath = SupportPath;
			return string.Empty;
		}

		/// <summary>Returns the value of the internal variable 'ThemePath'</summary>
		/// <returns>The value of ThemePath</returns>
		public string GetThemePath()
		{
			return ThemePath;
		}

		/// <summary>Returns the value of the internal variable 'SupportPath'</summary>
		/// <returns>The value of SupportPath</returns>
		public string GetSupportPath()
		{
			return SupportPath;
		}

		/// <summary>Returns the physical path and file name of RelativePathFile.
		/// <para>Checks if ThemePath + RelativePathFile exists first, if so, returns that physical path and filename.</para>
		/// <para>If RelativePathFile exists, returns that.</para>
		/// <para>If SupportPath + RelativePathFile exists, returns that.</para>
		/// <para>Otherwise returns the string "unknown.file"</para>
		/// </summary>
		/// <param name="RelativePathFile">The relative path and file to retrieve (such as 'xsl/Appearence.xsl')</param>
		/// <returns>The physical path and file found</returns>
		public string FilePath(string RelativePathFile)
		{
			if (c == null)
				c = HttpContext.Current;

			RelativePathFile = RelativePathFile.Replace(DefaultThemePath, string.Empty);
			RelativePathFile = RelativePathFile.Replace(DefaultThemePath.Replace("~/", string.Empty), string.Empty);

			string ret = c.Server.MapPath(ThemePath + RelativePathFile);
			if (File.Exists(ret))
				return ret;
			ret = c.Server.MapPath(DefaultThemePath + RelativePathFile);
			if (File.Exists(ret))
				return ret;
			ret = c.Server.MapPath(RelativePathFile);
			if (File.Exists(ret))
				return ret;
			ret = c.Server.MapPath("~/" + RelativePathFile);
			if (File.Exists(ret))
				return ret;
			ret = c.Server.MapPath(SupportPath + RelativePathFile);
			if (File.Exists(ret))
				return ret;
			return "unknown.file";
		}


		/// <summary>Returns the virtual path and file name of RelativePathFile.
		/// <para>Checks if ThemePath + RelativePathFile exists first, if so, returns that virtual path and filename.</para>
		/// <para>If RelativePathFile exists, returns that.</para>
		/// <para>If SupportPath + RelativePathFile exists, returns that.</para>
		/// <para>Otherwise returns the string "unknown.file"</para>
		/// </summary>
		/// <param name="RelativePathFile">The relative path and file to retrieve (such as 'xsl/Appearence.xsl')</param>
		/// <returns>The virtual path and file found</returns>
		public string FileUrl(string RelativePathFile)
		{
			string ret = "unknown.file";
			if (c == null)
				c = HttpContext.Current;

			RelativePathFile = RelativePathFile.Replace(DefaultThemePath, string.Empty);
			RelativePathFile = RelativePathFile.Replace(DefaultThemePath.Replace("~/", string.Empty), string.Empty);

			string filePath = c.Server.MapPath(ThemePath + RelativePathFile);
			string VDirPath;

			if (File.Exists(filePath))
				ret = ThemePath + RelativePathFile;
			else
			{
				filePath = c.Server.MapPath(DefaultThemePath + RelativePathFile);
				if (File.Exists(filePath))
					ret = DefaultThemePath + RelativePathFile;
				else
				{
					filePath = c.Server.MapPath(RelativePathFile);
					if (File.Exists(filePath))
					{
						ret = Token.After(c.Request.Url.AbsoluteUri, 4, "/");
						ret = Token.First(Token.Before(ret, Token.Count(ret, "/"), "/"), "?");
						ret = ret + "/" + RelativePathFile;
					}
					else
					{
						filePath = c.Server.MapPath("~/" + RelativePathFile);
						if (File.Exists(filePath))
						{
							ret = Token.After(c.Request.Url.AbsoluteUri, 4, "/");
							ret = Token.First(Token.Before(ret, Token.Count(ret, "/"), "/"), "?");
							ret = ret + "/" + RelativePathFile;
						}
						else
						{
							filePath = c.Server.MapPath(SupportPath + RelativePathFile);
							if (File.Exists(filePath))
							{
								ret = SupportPath + RelativePathFile;
								VDirPath = Token.First(Token.Before(c.Request.Url.AbsoluteUri, 4, "/"), "?");
								ret = ret.Replace("~/", string.Empty);
								if (ret.StartsWith("/"))
									ret = VDirPath + ret;
								else
									ret = VDirPath + "/" + ret;
								return ret;
							}
						}
					}
				}
			}
			VDirPath = Token.First(Token.Before(c.Request.Url.AbsoluteUri, 5, "/"), "?");
			ret = ret.Replace("~/", string.Empty);
			if (ret.StartsWith("/"))
				ret = VDirPath + ret;
			else
				ret = VDirPath + "/" + ret;
			return ret;
		}


		/// <summary>Uses FilePath to determine the path to a physical file and then retrieves the string contents.</summary>
		/// <param name="RelativePathFile">The relative path and file to retrieve</param>
		/// <returns>The contents of the file</returns>
		public string FileContents(string RelativePathFile)
		{
			RelativePathFile = FilePath(RelativePathFile);
			if (RelativePathFile != "unknown.file")
				return MetX.IO.FileSystem.FileToString(RelativePathFile);
			return string.Empty;
		}

		/// <summary>
		/// Same as calling System.IO.File.Exists
		/// </summary>
		/// <param name="PathAndFilename">Path and filename to the file</param>
		/// <returns>True if the file exists</returns>
		public bool FileExists(string PathAndFilename)
		{
			return File.Exists(PathAndFilename);
		}


		/// <summary>Same as calling System.IO.Directory.Exists</summary>
		/// <param name="Path">The path to test existence for</param>
		/// <returns>True if the folder exists</returns>
		public bool DirectoryExists(string Path)
		{
			return Directory.Exists(Path);
		}

		/// <summary>Same as calling System.IO.Directory.Create</summary>
		/// <param name="Path">The path to create</param>
		/// <returns>True if the folder exists</returns>
		public string CreateDirectory(string Path)
		{
			if (!Directory.Exists(Path))
				Directory.CreateDirectory(Path);
			return string.Empty;
		}

		/// <summary>Same as calling System.IO.Directory.Create</summary>
		/// <param name="Path">The path to create</param>
		/// <returns>True if the folder exists</returns>
		public bool InsureDirectory(string Path)
		{
			if (!Directory.Exists(Path))
			{
				Directory.CreateDirectory(Path);
				return true;
			}
			return false;
		}

		/// <summary>
		/// Appends a string to an internal string builder
		/// </summary>
		/// <param name="sbVarName">The name of the string builder to append to</param>
		/// <param name="ToAppend">The string to append</param>
		/// <returns>An empty string</returns>
		public string sbAppend(string sbVarName, string ToAppend)
		{
			if (sbVars == null)
				sbVars = new Dictionary<string, StringBuilder>();
			if (sbVars.ContainsKey(sbVarName))
				sbVars[sbVarName].Append(ToAppend);
			else
				sbVars[sbVarName] = new StringBuilder(ToAppend);
			return string.Empty;
		}


		/// <summary>
		/// Appends a string to an internal string builder with a new line character at the end
		/// </summary>
		/// <param name="sbVarName">The name of the string builder to append to</param>
		/// <param name="ToAppend">The string to append</param>
		/// <returns>An empty string</returns>
		public string sbAppendLine(string sbVarName, string ToAppend)
		{
			if (sbVars == null)
				sbVars = new Dictionary<string, StringBuilder>();
			if (!sbVars.ContainsKey(sbVarName))
				sbVars[sbVarName] = new StringBuilder();
			sbVars[sbVarName].AppendLine(ToAppend);
			return string.Empty;
		}


		/// <summary>
		/// Retrieves the contents of an internal string builder
		/// </summary>
		/// <param name="sbVarName">The string builder to retrieve</param>
		/// <returns>The string builder contents</returns>
		public string sbGetVar(string sbVarName)
		{
			if (sbVars != null && sbVars.ContainsKey(sbVarName))
				return sbVars[sbVarName].ToString();
			return string.Empty;
		}


		/// <summary>
		/// Removes an internal string builder
		/// </summary>
		/// <param name="sbVarName">The string builder to remove</param>
		/// <returns>An empty string</returns>
		public string sbRemove(string sbVarName)
		{
			if (sbVars != null && sbVars.ContainsKey(sbVarName))
				sbVars.Remove(sbVarName);
			return string.Empty;
		}


		/// <summary>Removes a specific internal variable set by SetVar</summary>
		/// <param name="VarName">The variable to remove</param>
		/// <returns>a blank string</returns>
		public string RemoveVar(string VarName)
		{
			if (Vars == null)
				Vars = new Dictionary<string, string>();
			if (Vars.ContainsKey(VarName))
				Vars.Remove(VarName);
			return string.Empty;
		}

		/// <summary>Removes a specific internal variable set by SetVar</summary>
		/// <param name="VarName">The variable to remove</param>
		/// <returns>a blank string</returns>
		public string R(string VarName)
		{
			if (Vars == null)
				Vars = new Dictionary<string, string>();
			if (Vars.ContainsKey(VarName))
				Vars.Remove(VarName);
			return string.Empty;
		}


		/// <summary>Clears all internal (non-xsl) variables</summary>
		/// <returns>a blank string</returns>
		public string ClearVars()
		{
			if (Vars != null)
				Vars.Clear();
			return string.Empty;
		}


		/// <summary>Sets an internal variable to some value. This value will persist until changed or until the end of rendering</summary>
		/// <param name="VarName">The variable name to set</param>
		/// <param name="VarValue">The value of the variable</param>
		/// <returns>a blank string</returns>
		public string SetVar(string VarName, string VarValue)
		{
			if (Vars == null)
				Vars = new Dictionary<string, string>();
			Vars[VarName] = VarValue;
			return string.Empty;
		}


		/// <summary>Returns the value of an internal variable or a blank string if that variable wasn't set.</summary>
		/// <param name="VarName">The variable to retrieve</param>
		/// <returns>The variables value or a blank string if not set</returns>
		public string GetVar(string VarName)
		{
			if (Vars == null)
				Vars = new Dictionary<string, string>();
			if (Vars.ContainsKey(VarName))
				return Vars[VarName];
			return string.Empty;
		}

		/// <summary>Returns the value of an internal variable or a blank string if that variable wasn't set.</summary>
		/// <param name="VarName">The variable to retrieve</param>
		/// <returns>The variables value or a blank string if not set</returns>
		public string GetAndRemoveVar(string VarName)
		{
			string ret = string.Empty;
			if (Vars != null && Vars.ContainsKey(VarName))
			{
				ret = Vars[VarName];
				Vars.Remove(VarName);
			}
			return ret;
		}

		/// <summary>Sets an internal variable to some value. This value will persist until changed or until the end of rendering</summary>
		/// <param name="VarName">The variable name to set</param>
		/// <param name="VarValue">The value of the variable</param>
		/// <returns>a blank string</returns>
		public string S(string VarName, string VarValue)
		{
			if (Vars == null)
				Vars = new Dictionary<string, string>();
			Vars[VarName] = VarValue;
			return string.Empty;
		}


		/// <summary>Returns the value of an internal variable or a blank string if that variable wasn't set.</summary>
		/// <param name="VarName">The variable to retrieve</param>
		/// <returns>The variables value or a blank string if not set</returns>
		public string G(string VarName)
		{
			if (Vars == null)
				Vars = new Dictionary<string, string>();
			if (Vars.ContainsKey(VarName))
				return Vars[VarName];
			return string.Empty;
		}

		/// <summary>Returns the value of an internal variable or a blank string if that variable wasn't set.</summary>
		/// <param name="VarName">The variable to retrieve</param>
		/// <returns>The variables value or a blank string if not set</returns>
		public string P(string VarName)
		{
			string ret = string.Empty;
			if (Vars != null && Vars.ContainsKey(VarName))
			{
				ret = Vars[VarName];
				Vars.Remove(VarName);
			}
			return ret;
		}


		/// <summary>Returns true if the variable has been set and if the value's length is greater than zero</summary>
		/// <param name="VarName">The variable to test.</param>
		/// <returns>True if the variable has been set to a non empty string</returns>
		public bool IsVarSet(string VarName)
		{
			if (Vars == null)
				Vars = new Dictionary<string, string>();
			if (Vars.ContainsKey(VarName) && Vars[VarName] != null && Vars[VarName].Length > 0)
				return true;
			return false;
		}


		/// <summary>Creates an incrementing number for the ToHash value and stores it in a table. Each time afterward sHash is called with ToHash, the same value will be returned. Among other things, useful for generating a script block and a short unique div name.</summary>
		/// <param name="ToHash">The item to hash</param>
		/// <returns>The hash value for the item</returns>
		public string sHash(string ToHash)
		{
			if (HT == null)
				HT = new Dictionary<string, string>();
			if (!HT.ContainsKey(ToHash))
			{
				HT.Add(ToHash, NextHash.ToString());
				NextHash += 1;
			}
			return HT[ToHash];
		}


		/// <summary>Replaces one string with another. Equivalent to calling string.Replace</summary>
		/// <param name="ToSearch">The text to do the replacement on</param>
		/// <param name="ToFind">The text to find</param>
		/// <param name="ToReplace">The text to replace</param>
		/// <returns>The string with text replaced.</returns>
		public string sReplace(string ToSearch, string ToFind, string ToReplace)
		{
			return ToSearch.Replace(ToFind, ToReplace);
		}


		/// <summary>Returns an xml string of the Inner text of one or more Nodes</summary>
		/// <param name="Nodes">The nodes to return InnerXml for</param>
		/// <returns>The inner xml string of the nodes</returns>
		public string InnerXML(XPathNodeIterator Nodes)
		{
			if (Nodes == null)
			{
				return string.Empty;
			}
			XPathNavigator NameValueNodes = null;
			Nodes.MoveNext();
			NameValueNodes = Nodes.Current;
			return NameValueNodes.InnerXml;
		}


		/// <summary>Returns an xml string of the Outer text of one or more Nodes</summary>
		/// <param name="Nodes">The nodes to return OuterXml for</param>
		/// <returns>The outer xml string of the nodes</returns>
		public string OuterXML(XPathNodeIterator Nodes)
		{
			if (Nodes == null)
			{
				return string.Empty;
			}
			XPathNavigator NameValueNodes = null;
			Nodes.MoveNext();
			NameValueNodes = Nodes.Current;
			return NameValueNodes.OuterXml;
		}


		/// <summary>Returns the OuterXml of Nodes in javascript format set to the javascript variable VarName. So this generates a single (usually very long) line of javascript.</summary>
		/// <param name="VarName">The javascript variable name to place the outerxml into</param>
		/// <param name="Nodes">The nodes to javascript code</param>
		/// <returns>One line of javascript of the form var VarName = "outerxml of Nodes";</returns>
		public string OuterXmlJSON(string VarName, XPathNodeIterator Nodes)
		{
			StringBuilder ret = new StringBuilder(OuterXML(Nodes));
			if (ret.Length > 0)
			{
				ret.Replace("\"", "\\\"");
				ret.Replace(System.Environment.NewLine, "\\n");
				ret.Replace("\r", "\\n");
				ret.Replace("\n", "\\n");
				ret.Insert(0, "var " + VarName + " = \"<?xml version=\\\"1.0\\\" encoding=\\\"UTF-8\\\" ?>\\n");
				ret.Append("\";");
			}
			return ret.ToString();
		}


		/// <summary>Returns the maximum of two dates</summary>
		/// <param name="sDate1">The first date to compare</param>
		/// <param name="sDate2">The second date to compare</param>
		/// <returns>The greater date</returns>
		public string dateMax(string sDate1, string sDate2)
		{
			if (sDate1.Length > 0)
			{
				if (sDate2.Length > 0)
				{
					System.DateTime Date1 = System.DateTime.Parse(sDate1);
					System.DateTime Date2 = System.DateTime.Parse(sDate2);
					if (Date1 > Date2)
						return sDate1;
					else
						return sDate2;
				}
				else
					return sDate1;
			}
			return sDate2;
		}


		/// <summary>Determins if one date is between or equal to two other dates</summary>
		/// <param name="sDateToTest">The date to test</param>
		/// <param name="sDateBegin">The lower boundary to test</param>
		/// <param name="sDateEnd">The upper boundary to test</param>
		/// <returns>true if sDateToTest is between sDateBegin and sDateEnd</returns>
		public bool dateIsInRange(string sDateToTest, string sDateBegin, string sDateEnd)
		{
			if (sDateToTest.Length > 0)
			{
				if (sDateBegin.Length > 0)
				{
					if (sDateEnd.Length > 0)
					{
						System.DateTime DateToTest = System.DateTime.Parse(sDateToTest);
						System.DateTime DateBegin = System.DateTime.Parse(sDateBegin);
						System.DateTime DateEnd = System.DateTime.Parse(sDateEnd);
						if (DateToTest >= DateBegin && DateToTest <= DateEnd)
						{
							return true;
						}
					}
				}
			}
			return false;
		}


		/// <summary>Retrieves the nth delimited token</summary>
		/// <param name="AllTokens">The string to retrieve a token from</param>
		/// <param name="N">The token number to retrieve</param>
		/// <param name="Delimiter">The delimiter separating each token</param>
		/// <returns>The requested token or a blank string</returns>
		public string GetToken(string AllTokens, int N, string Delimiter)
		{
			return Token.Get(AllTokens, N, Delimiter);
		}


		/// <summary>Returns the index of a string inside another string. Equivalent to string.IndexOf</summary>
		/// <param name="ToSearch">The string to search</param>
		/// <param name="ToFind">The string to find</param>
		/// <returns>The index of ToFind in ToSearch. -1 if nothing is found</returns>
		public int IndexOf(string ToSearch, string ToFind)
		{
			return ToSearch.IndexOf(ToFind);
		}

		public bool Contains(string ToSearch, string ToFind)
		{
			return ToSearch.Contains(ToFind);
		}

		public bool StartsWith(string ToSearch, string ToFind)
		{
			return ToSearch.StartsWith(ToFind);
		}

		public bool EndsWith(string ToSearch, string ToFind)
		{
			return ToSearch.EndsWith(ToFind);
		}

		/// <summary>Determins if the date passed in is today regardless of the time of day</summary>
		/// <param name="DateStringToTest">The date to test</param>
		/// <returns>true if the date is between 00:00am and 11:59pm today</returns>
		public bool IsToday(string DateStringToTest)
		{
			if (Microsoft.VisualBasic.Information.IsDate(DateStringToTest))
			{
				return (System.Convert.ToDateTime(DateStringToTest) >= System.DateTime.Today) & (System.Convert.ToDateTime(DateStringToTest) < System.DateTime.Today.AddDays(1));
			}
			else
			{
				return false;
			}
		}

		/// <summary>Determines if a date is in the past</summary>
		/// <param name="DateStringToTest">The date to test</param>
		/// <returns>true if the date is in the past (even by a second)</returns>
		public bool IsInThePast(string DateStringToTest)
		{
			if (Microsoft.VisualBasic.Information.IsDate(DateStringToTest))
				return Convert.ToDateTime(DateStringToTest).CompareTo(DateTime.Now) < 0;
			return false;
		}

		/// <summary>Returns the proper case of a string (such as a name). So "this is a test" becomes "This Is A Test".</summary>
		/// <param name="Text">The text to proper case</param>
		/// <returns>The proper case string</returns>
		public string ProperCase(string Text)
		{
			string ret = string.Empty;
			try
			{
				ret = Microsoft.VisualBasic.Strings.StrConv(Text, Microsoft.VisualBasic.VbStrConv.ProperCase, 0);
			}
			catch { }
			return ret;
		}


		/// <summary>Returns the current date and time</summary>
		/// <returns>The current date/time</returns>
		public string Today()
		{
			return System.DateTime.Now.ToString();
		}


		/// <summary>Returns the integer representation of a string</summary>
		/// <param name="sOriginalText">The string to convert</param>
		/// <returns>The integer representation of a string</returns>
		public int nzInteger(string sOriginalText)
		{
			return Worker.nzInteger(sOriginalText);
		}

		/// <summary>Returns the double representation of a string</summary>
		/// <param name="sOriginalText">The string to convert</param>
		/// <returns>The double representation of a string</returns>
		public double nzDouble(string sOriginalText)
		{
			return Worker.nzDouble(sOriginalText);
		}


		/// <summary>Extracts a name from an email address</summary>
		/// <param name="sOriginalText">The Email address</param>
		/// <returns>The extracted proper case name</returns>
		public string EmailToName(string sOriginalText)
		{
			return Worker.EmailToName(sOriginalText, string.Empty);
		}


		/// <summary>Coverts the passed string to lowercase</summary>
		/// <param name="sOriginalText">The text to convert</param>
		/// <returns>The lowercase version of the string</returns>
		public string Lower(string sOriginalText)
		{
			return Worker.nzString(sOriginalText).ToLower();
		}


		/// <summary>Coverts the passed string to uppercase</summary>
		/// <param name="sOriginalText">The text to convert</param>
		/// <returns>The uppercase version of the string</returns>
		public string Upper(string sOriginalText)
		{
			return Worker.nzString(sOriginalText).ToUpper();
		}

		/// <summary>Returns the number of days old the date passed in is or a blank string if it isn't a valid date</summary>
		/// <param name="XmlDate">The date to calculate</param>
		/// <returns>The number of days old the date is</returns>
		public string sXmlAgeOfDate(string XmlDate)
		{
			XmlDate = System.Convert.ToString(XmlDate + string.Empty).Trim();
			if (XmlDate.Length > 0)
			{
				System.DateTime dt = System.Convert.ToDateTime(XmlDate);
				dt = dt.AddHours(-dt.Hour).AddMinutes(-dt.Minute);
				TimeSpan ts = DateTime.Today.Subtract(dt);
				return ts.Days.ToString();
			}
			return string.Empty;
		}

		/// <summary>Converts an xml date/time into a javascript compatible date</summary>
		/// <param name="XmlDate">The date/time to convert</param>
		/// <param name="DefaultValue">The value to return if the date passed is blank or invalid</param>
		/// <returns>The javascript date in MMMM, dd YYYY format</returns>
		public string sXmlDateOnlyForJavaScript(string XmlDate, string DefaultValue)
		{
			XmlDate = System.Convert.ToString(XmlDate + string.Empty).Trim();
			if (XmlDate.Length > 0 & Microsoft.VisualBasic.Information.IsDate(XmlDate))
			{
				return System.Convert.ToDateTime(XmlDate).ToString("MMMM, dd yyyy");
			}
			return DefaultValue;
		}


		/// <summary>Converts an xml date/time into a displayable date (MM/dd/YYYY format)</summary>
		/// <param name="XmlDate">The date to convert</param>
		/// <returns>The displayable date</returns>
		public string sXmlDateOnly(string XmlDate)
		{
			XmlDate = System.Convert.ToString(XmlDate + string.Empty).Trim();
			if (XmlDate.Length > 0 & Microsoft.VisualBasic.Information.IsDate(XmlDate))
			{
				return System.Convert.ToDateTime(XmlDate).ToString("MM/dd/yyyy").ToLower().Replace("01/01/1900", string.Empty);
			}
			return string.Empty;
		}


		/// <summary>Converts an xml date/time into a displayable date/time value ("MM/dd/YYYY hh:mm tt" format)</summary>
		/// <param name="XmlDate">The date to convert</param>
		/// <returns>The displayable date</returns>
		public string sXmlDate(string XmlDate)
		{
			XmlDate = System.Convert.ToString(XmlDate + string.Empty).Trim();
			if (XmlDate.Length > 0)
			{
				return System.Convert.ToDateTime(XmlDate).ToString("MM/dd/yyyy hh:mm tt").ToLower().Replace(" 12:00 am", string.Empty);
			}
			return string.Empty;
		}


		/// <summary>Formats a number to a particular format (see the VB Format() function).</summary>
		/// <param name="Value">The value to format</param>
		/// <param name="FormatString">The VB.NET format string</param>
		/// <returns>The formated string</returns>
		public string Format(float Value, string FormatString)
		{
			return Microsoft.VisualBasic.Strings.Format(Value, FormatString);
		}


		/// <summary>Converts an xml date/time into a displayable date/time value ("MM/dd/YYYY hh:mm tt" format)</summary>
		/// <param name="XmlDate">The date to convert</param>
		/// <param name="sFormat">The VB.NET format string to format the date/time to</param>
		/// <returns>The formated date/time string</returns>
		public string sXmlDate(string XmlDate, string sFormat)
		{
			XmlDate = System.Convert.ToString(XmlDate + string.Empty).Trim();
			if (XmlDate.Length > 0)
			{
				return System.Convert.ToDateTime(XmlDate).ToString(sFormat).ToLower();
			}
			return string.Empty;
		}

		/// <summary>Returns the first name (word) from the given string</summary>
		/// <param name="Name">The name to parse</param>
		/// <returns>The first name found</returns>
		public string FirstName(string Name)
		{
			return Token.First(Name, " ");
		}

		/// <summary>Returns the last name (word) from the given string</summary>
		/// <param name="Name">The name to parse</param>
		/// <returns>The last name found</returns>
		public string LastName(string Name)
		{
			return Token.Last(Name, " ");
		}


		/// <summary>Increments an internal counter (NextID) and returns that value.</summary>
		/// <returns>The next higher ID</returns>
		public string GetNextID()
		{
			NextID += 1;
			return NextID.ToString();
		}


		/// <summary>Subtracts one from the internal layer count (starting at 10,000) and returns that value</summary>
		/// <returns>The next lower layer</returns>
		public string GetNextLayer()
		{
			NextLayer -= 1;
			return NextLayer.ToString();
		}


		/// <summary>Occilates between returning "contentDataRow1" and "contentDataRow2"</summary>
		/// <returns>The next row CSS class value</returns>
		public string GetNextRowClass()
		{
			string t = CurrRowClass;
			CurrRowClass = NextRowClass;
			NextRowClass = t;
			return t;
		}

		/// <summary>
		/// Clears the internal list of totals
		/// </summary>
		/// <returns></returns>
		public string ClearTotals()
		{
			RunningTotals = new Dictionary<string, double>();
			return string.Empty;
		}

		/// <summary>Clears the internal total to 0</summary>
		/// <param name="TotalName">The name of the total to clear</param>
		/// <returns>The string "0"</returns>
		public string ClearTotal(string TotalName)
		{
			return ClearTotal(TotalName, "0");
		}

		/// <summary>Clears the internal total to some intial value</summary>
		/// <param name="TotalName">The name of the total to clear</param>
		/// <param name="sInitialValue">The value to set internal total to</param>
		/// <returns>an empty string</returns>
		public string ClearTotal(string TotalName, string sInitialValue)
		{
			if (RunningTotals == null)
				RunningTotals = new Dictionary<string, double>();
			if (RunningTotals.ContainsKey(TotalName))
				RunningTotals[TotalName] = Worker.nzDouble(sInitialValue);
			else
				RunningTotals.Add(TotalName, Worker.nzDouble(sInitialValue));
			return string.Empty;
		}


		/// <summary>Adds a value to the internal total</summary>
		/// <param name="TotalName">The name of the total</param>
		/// <param name="ToAdd">The value to add to the internal total</param>
		/// <returns>An empty string</returns>
		public string AddToTotal(string TotalName, string ToAdd)
		{
			if (RunningTotals == null)
				RunningTotals = new Dictionary<string, double>();
			if (RunningTotals.ContainsKey(TotalName))
				RunningTotals[TotalName] += nzDouble(ToAdd);
			else
				RunningTotals.Add(TotalName, Worker.nzDouble(ToAdd));
			return string.Empty;
		}


		/// <summary>Subtracts a value from the internal total</summary>
		/// <param name="TotalName">The name of the total</param>
		/// <param name="ToSubtract">The amount to subtract</param>
		/// <returns>a blank string</returns>
		public string SubtractFromTotal(string TotalName, string ToSubtract)
		{
			if (RunningTotals == null)
				RunningTotals = new Dictionary<string, double>();
			if (RunningTotals.ContainsKey(TotalName))
				RunningTotals[TotalName] -= nzDouble(ToSubtract);
			else
				RunningTotals.Add(TotalName, -Worker.nzDouble(ToSubtract));
			return string.Empty;
		}


		/// <summary>Returns the current internal total value</summary>
		/// <param name="TotalName">The name of the total</param>
		/// <param name="DecimalPlaces">The number of decimal places to return the internal total (normally 2 or 0)</param>
		/// <returns>The internal total formatted to the number of decimal places</returns>
		public string GetTotal(string TotalName, int DecimalPlaces)
		{
			if (RunningTotals == null)
				RunningTotals = new Dictionary<string, double>();
			if (DecimalPlaces > 0)
				if (RunningTotals.ContainsKey(TotalName))
					return RunningTotals[TotalName].ToString("0." + new String('0', DecimalPlaces));
				else
					return "0." + new String('0', DecimalPlaces);
			else if (RunningTotals.ContainsKey(TotalName))
				return RunningTotals[TotalName].ToString();
			return "0";
		}

		/// <summary>
		/// Returns a distinct set of nodes
		/// </summary>
		/// <param name="nodeset"></param>
		/// <returns></returns>
		public XPathNavigator[] Distinct(XPathNodeIterator nodeset)
		{
			if (nodeset.Count == 0)
				return new XPathNavigator[0];
			Dictionary<string, XPathNavigator> retNodes = new Dictionary<string, XPathNavigator>();
			while (nodeset.MoveNext())
				if (!retNodes.ContainsKey(nodeset.Current.Value))
					retNodes.Add(nodeset.Current.Value, nodeset.Current);
			XPathNavigator[] ret = new XPathNavigator[retNodes.Count];
			retNodes.Values.CopyTo(ret, 0);
			return ret;
		}

		#region Dealing with: <Hints>
		
		XPathNavigator _HintsParent;
		string _HintsFor;
		string _HintsForName;
		XPathNavigator _CurrHint;

		public string SetupHints(XPathNodeIterator ParentNodeToTargettedHints)
		{
			if(ParentNodeToTargettedHints != null && ParentNodeToTargettedHints.Count > 0)
			{
				XPathNodeIterator t = ParentNodeToTargettedHints.Clone();
				t.MoveNext();
				_HintsParent = t.Current;
			}
			else
				_HintsParent = null;
			return string.Empty;
		}

		public string HintsFor(string For)
		{
			_HintsFor = For;
			_HintsForName = null;
			_CurrHint = null;
			return string.Empty;
		}

		public string HintName(string Name)
		{
			return H(_HintsFor, Name, string.Empty);
		}

		public string HS(string Name)
		{
			if (string.IsNullOrEmpty(_HintsFor))
				return string.Empty;
			if (Name != _HintsForName)
				return H(_HintsFor, Name, "Skip");
			return Hint("Skip");
		}

		public string H()
		{
			return H("Skip");
		}

		public string H(string Name, string AttributeValue)
		{
			if(!string.IsNullOrEmpty(_HintsFor ))
				return H(_HintsFor, Name, AttributeValue);
			return string.Empty;
		}

		public string H(string For, string Name, string AttributeName) 
		{
			string ret = string.Empty;
			_HintsFor = For;
			_HintsForName = Name;
			_CurrHint = _HintsParent.SelectSingleNode("Hints[@For=\"" + xml.AttributeEncode(_HintsFor) + "\"]/Hint[@Name=\"" + xml.AttributeEncode(_HintsForName) + "\"]");
			if (_CurrHint != null)
				if (!string.IsNullOrEmpty(AttributeName))
					ret = Worker.nzString(_CurrHint.GetAttribute(AttributeName, string.Empty));
			return ret;
		}

		public string H(string AttributeName)
		{
			if (_CurrHint != null && !string.IsNullOrEmpty(AttributeName))
				return Worker.nzString(_CurrHint.GetAttribute(AttributeName, string.Empty));
			return string.Empty;
		}
		public string Hint(string AttributeName)
		{
			if (_CurrHint != null && !string.IsNullOrEmpty(AttributeName))
				return Worker.nzString(_CurrHint.GetAttribute(AttributeName, string.Empty));
			return string.Empty;
		}

		public string ResolveHint(string Name, string Default)
		{
			string ret = string.Empty;
			if (!string.IsNullOrEmpty(_HintsFor))
			{
				if (Name != _HintsForName)
					ret = H(_HintsFor, Name, "Skip");
				else
					ret = H("Skip");

				if (string.IsNullOrEmpty(ret))
					ret = Default;
				else if (Worker.nzBoolean(ret) == true)
					ret = string.Empty;
				else
				{
					ret = H("Code");
					if (string.IsNullOrEmpty(ret))
						ret = Default;
				}
			}
			return ret;
		}
		#endregion
	}

}
